library www_api;

import 'dart:io';
import 'dart:async';
import 'dart:convert';



class WebReflect {
  /// 根节点id
  static const String rootId = 'root';

  Function _eventListener;
  WebSocket _websocket;

  /// 通过导入一个WebSocket对象来新建一个WebReflect
  WebReflect(this._websocket) {
    _websocket.listen(this._handler);
  }

  /// 修改目标网页dom
  /// 例子:
  /// ```
  /// // 初始化root节点
  /// reflect.set('root', '<div id="output"></div>');
  /// // 为output设置子节点
  /// reflect.set('output', '<h1>Hello!</h1>');
  /// ```
  void set(String id, String value) {
    _websocket.add(JSON.encode({
      'type': 'set',
      'id': id,
      'value': value
    }));
  }

  /// 为目标网页添加子节点
  /// 例子:
  /// ```
  /// // 为root设置子节点
  /// reflect.modify('root', '<h1>Hello!</h1>');
  /// ```
  void append(String id, String value) {
    _websocket.add(JSON.encode({
      'type': 'append',
      'id': id,
      'value': value
    }));
  }

  /// 在目标网页中执行js
  /// 例子:
  /// ```
  /// // alert
  /// reflect.executeJs('alert("hello webreflect!")');
  /// ```
  void executeJs(String value) {
    _websocket.add(JSON.encode({
      'type': 'javascript',
      'value': value
    }));
  }

  /// 中断Websocket连接
  /// 例子:
  /// ```
  /// reflect.disconnect();
  /// ```
  Future<dynamic> disconnect() async {
    return await _websocket.close();
  }

  /// 事件处理器, 例子:
  /// ```
  /// reflect.addListener((m) {
  ///   m.forEach((id, attr) {
  ///     // do something
  ///   });
  /// });
  /// ```
  void setListener(void onData(Map<String, String> msg)) {
    _eventListener = onData;
  }

  _handler(rawMsg) {
    var decoded = JSON.decode(rawMsg);
    _eventListener(decoded);
  }
}