// Copyright (c) 2017, kari. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'dart:html';
import 'dart:convert';
import 'dart:js' show context;

Map<String, String> buffer = {};
WebSocket websocket;

void main() {
  context['wrConnect'] = (address) {
    websocket = new WebSocket('ws://$address');
    websocket.onMessage.listen(handler);
  };

  context['wrBuffer'] = (id, value) {
    buffer[id] = value;
  };
  
  context['wrCommitBuffer'] = () {
    buffer.forEach((id, value) {
      websocket.sendString(JSON.encode({
        'type': 'commit',
        'id': id,
        'value': value
      }));
    });
    buffer.clear();
  };

  context['wrEvent'] = (id, eventname) {
    websocket.sendString(JSON.encode({
      'type': 'event',
      'id': id,
      'value': eventname
    }));
  };
}

void handler(MessageEvent e) {
  var msg = JSON.decode(e.data);

  var type  = msg['type'];
  var id    = msg['id'];
  var value = msg['value'];

  switch (type) {
    case 'set':
      context.callMethod('setHTML', [id, value]);
      break;
    case 'append':
      context.callMethod('appendHTML', [id, value]);
      break;
    case 'javascript':
      context.callMethod('eval', [value]);
      break;
    default:
      log('未知消息类型$type');
  }
}

void log(String message) {
  context['console'].callMethod('log', [message]);
}